package org.eclipse.palamedes.gdl.core.ast;

import org.eclipse.palamedes.kif.core.ast.KIFForm;
import org.eclipse.palamedes.kif.core.ast.KIFImplication;
import org.eclipse.palamedes.kif.core.ast.KIFLogConst;
import org.eclipse.palamedes.kif.core.ast.KIFRelSent;
import org.eclipse.palamedes.kif.core.ast.KIFSent;
import org.eclipse.palamedes.kif.core.ast.KIFSeq;
import org.eclipse.palamedes.kif.core.ast.KIFTerm;

public class GDLAST extends org.eclipse.palamedes.kif.core.ast.AST {

    final static String error1 = "Only an atomic sentence is allowed to be " +
                                 "the head of a rule";

    /**
     * Returns true if the given string is a GDL keyword; otherwise false.
     */
    public static boolean isKeyword(String symbol) {
    	return "true".equalsIgnoreCase    (symbol) ||
               "init".equalsIgnoreCase    (symbol) ||
               "distinct".equalsIgnoreCase(symbol) ||
               "legal".equalsIgnoreCase   (symbol) ||
               "does".equalsIgnoreCase    (symbol) ||
               "next".equalsIgnoreCase    (symbol) ||
               "goal".equalsIgnoreCase    (symbol) ||
               "role".equalsIgnoreCase    (symbol) ||
               "terminal".equalsIgnoreCase(symbol);    	
    }
    
    /*
     * (non-Javadoc)
     * @see org.eclipse.palamedes.kif.core.ast.AST#newForm()
     */
    @Override
    public KIFForm newForm() {
        return new GDLForm(super.newForm());
    }

    /*
     * (non-Javadoc)
     * @see org.eclipse.palamedes.kif.core.ast.AST#newForm(org.eclipse.palamedes.kif.core.ast.KIFSeq)
     */
    @Override
    public KIFForm newForm(KIFSeq<KIFSent> sentences) {
        return new GDLForm(super.newForm(sentences));
    }

    /*
     * (non-Javadoc)
     * @see org.eclipse.palamedes.kif.core.ast.AST#newImplication(org.eclipse.palamedes.kif.core.ast.KIFSent)
     */
    @Override
    public KIFImplication newImplication(KIFSent sentence) {

        // correction for KIF parser, e.g. terminal (nullary relation)
        if (sentence instanceof KIFLogConst)
            sentence = newRelSent(sentence.getName(), null);

        if ("goal".equalsIgnoreCase(sentence.getName()))
            return new RuleGoal(super.newImplication(sentence));

        else if ("terminal".equalsIgnoreCase(sentence.getName()))
            return new RuleTerminal(super.newImplication(sentence));

        else if ("next".equalsIgnoreCase(sentence.getName()))
            return new RuleNext(super.newImplication(sentence));

        else if ("legal".equalsIgnoreCase(sentence.getName()))
            return new RuleLegal(super.newImplication(sentence));

        return new RuleGeneral(super.newImplication(sentence));
    }

    /*
     * (non-Javadoc)
     * @see org.eclipse.palamedes.kif.core.ast.AST#newImplication(org.eclipse.palamedes.kif.core.ast.KIFSent, org.eclipse.palamedes.kif.core.ast.KIFSeq)
     */
    @Override
    public KIFImplication newImplication(KIFSent sentence,
                                         KIFSeq<KIFSent> sequence) {

        // correction for KIF parser, e.g. terminal (nullary relation)
        if (sentence instanceof KIFLogConst)
            sentence = newRelSent(sentence.getName(), null);

        if ("goal".equalsIgnoreCase(sentence.getName()))
            return new RuleGoal(super.newImplication(sentence, sequence));

        else if ("terminal".equalsIgnoreCase(sentence.getName()))
            return new RuleTerminal(super.newImplication(sentence, sequence));

        else if ("next".equalsIgnoreCase(sentence.getName()))
            return new RuleNext(super.newImplication(sentence, sequence));

        else if ("legal".equalsIgnoreCase(sentence.getName())) {
            return new RuleLegal(super.newImplication(sentence, sequence));
        }

        return new RuleGeneral(super.newImplication(sentence, sequence));
    }

    /*
     * (non-Javadoc)
     * @see org.eclipse.palamedes.kif.core.ast.AST#newRelSent(java.lang.String, org.eclipse.palamedes.kif.core.ast.KIFSeq)
     */
    @Override
    public KIFRelSent newRelSent(String constName, KIFSeq<KIFTerm> terms) {
        if ( "init".equalsIgnoreCase(constName) )
            return new Init( super.newRelSent(constName, terms) );

        if ( "does".equalsIgnoreCase(constName) )
            return new Does( super.newRelSent(constName, terms) );

        if ( "role".equalsIgnoreCase(constName) )
            return new Role( super.newRelSent(constName, terms) );

        if ( "true".equalsIgnoreCase(constName) )
            return new True( super.newRelSent(constName, terms) );

        return super.newRelSent(constName, terms);
    }

}
