package org.eclipse.palamedes.gdl.core.ast;

import org.eclipse.palamedes.kif.core.ast.KIFForm;
import org.eclipse.palamedes.kif.core.ast.KIFObjConst;
import org.eclipse.palamedes.kif.core.ast.KIFSent;
import org.eclipse.palamedes.kif.core.ast.KIFSeq;
import org.eclipse.palamedes.kif.core.ast.KIFTerm;

public class GDLForm extends KIFForm {

    GDLForm(KIFForm form) {
        super(form.getAST());
        copyMembers(form);
    }



    /**
     * This method returns the function terms of a init predicate.
     *
     * @return Array of name strings
     */
    public KIFSeq<KIFTerm> getInitArguments() {
        KIFSeq<KIFTerm> initArguments = new KIFSeq<KIFTerm>();
        for (Init init : getRawInits())
        	initArguments.add( init.getArgument() );
        return initArguments;
    }


    /**
     * This method returns all init sentences.
     *
     * @return
     */
    public KIFSeq<Init> getRawInits() {
        KIFSeq<Init> rawInits = new KIFSeq<Init>();
        for (KIFSent sentence : getSentences())
            if (sentence instanceof Init)
                rawInits.add((Init) sentence);
        return rawInits;
    }


    /**
     * This method returns all role sentences.
     *
     * @return
     */
    public KIFSeq<Role> getRawRoles() {
        KIFSeq<Role> rawRoles = new KIFSeq<Role>();
        for (KIFSent sentence : getSentences())
            if (sentence instanceof Role)
                rawRoles.add((Role) sentence);
        return rawRoles;
    }

    public KIFSeq<RuleGeneral> getRawRuleGeneral() {
        KIFSeq<RuleGeneral> rawList = new KIFSeq<RuleGeneral>();
        for (KIFSent sentence : getSentences())
            if (sentence instanceof RuleGeneral)
                rawList.add((RuleGeneral) sentence);
        return rawList;
    }

    public KIFSeq<KIFSent> getRawGeneral() {
        KIFSeq<KIFSent> rawList = new KIFSeq<KIFSent>();
        rawList.addAll( getRawRuleGeneral() );
        rawList.addAll( getRawOthers()      );
        return rawList;
    }

    public KIFSeq<RuleGoal> getRawRuleGoal() {
        KIFSeq<RuleGoal> rawList = new KIFSeq<RuleGoal>();
        for (KIFSent sentence : getSentences())
            if (sentence instanceof RuleGoal)
                rawList.add((RuleGoal) sentence);
        return rawList;
    }

    public KIFSeq<RuleLegal> getRawRuleLegal() {
        KIFSeq<RuleLegal> rawList = new KIFSeq<RuleLegal>();
        for (KIFSent sentence : getSentences())
            if (sentence instanceof RuleLegal)
                rawList.add((RuleLegal) sentence);
        return rawList;
    }

    public KIFSeq<RuleNext> getRawRuleNext() {
        KIFSeq<RuleNext> rawList = new KIFSeq<RuleNext>();
        for (KIFSent sentence : getSentences())
            if (sentence instanceof RuleNext)
                rawList.add((RuleNext) sentence);
        return rawList;
    }

    public KIFSeq<RuleTerminal> getRawRuleTerminal() {
        KIFSeq<RuleTerminal> rawList = new KIFSeq<RuleTerminal>();
        for (KIFSent sentence : getSentences())
            if (sentence instanceof RuleTerminal)
                rawList.add((RuleTerminal) sentence);
        return rawList;
    }

    public KIFSeq<KIFSent> getRawOthers() {
        KIFSeq<KIFSent> rawList = new KIFSeq<KIFSent>();
        for (KIFSent sentence : getSentences())
            if ( !(sentence instanceof Rule) &&
                 !(sentence instanceof Does) &&
                 !(sentence instanceof Init) &&
                 !(sentence instanceof Role) &&
                 !(sentence instanceof True) )
                rawList.add(sentence);
        return rawList;
    }

    /**
     * This method returns the names of the roles.
     *
     * @return Array of name strings
     */
    public String[] getRoleNames() {
        KIFSeq<Role> rawRoles = getRawRoles();
        String[] roleNames = new String[rawRoles.size()];
        for (int i = 0; i < rawRoles.size(); i++)
            roleNames[i] = rawRoles.get(i).getRoleName();
        return roleNames;
    }

    /**
     * This method returns all role object constants.
     *
     * @return
     */
    public KIFSeq<KIFObjConst> getRoleObjConst() {
        KIFSeq<KIFObjConst> roleObjConst = new KIFSeq<KIFObjConst>();
        for (KIFSent sentence : getSentences())
            if (sentence instanceof Role)
                roleObjConst.add( (KIFObjConst) sentence.getOperands().get(0) );
        return roleObjConst;
    }

}
