package org.eclipse.palamedes.gdl.core.model;


/**
 * A game node is a unique instance of a game state within a game node tree.
 * 
 * Usually this interface is only implemented if you provide an own game tree
 * implementation.
 */
public interface IGameNode {

    /**
     * Returns the parent node or null if none exist.
     */
    public IGameNode getParent();

    /**
     * Returns the associated game state if it is still in memory.
     * 
     * getState() may return null after compacting the game tree
     * (which happens automatically) unless the nodes isPreserved() method returns true. 
     * IGame.regenerateNode(node) will restore the state in this case.
     * 
     * @see IGameNode#isPreserved()
     * @see IGameNode#setPreserve(boolean preserve)
     * @see IGame#regenerateNode(IGameNode node)
     */
    public IGameState getState();

    /**
     * Sets the associated game state
     */
    public void setState(IGameState state);

    /**
     * Returns the depth of the node in the game tree where 0 is the initial state.
     */
    public int getDepth();

    /**
     * Returns the moves that have lead to this game node.
     */
    public IMove[] getMoves();

    /**
     * Returns if this game nodes content should be preserved even in the case
     * of compacting the game tree.
     */
    public boolean isPreserved();

    /**
     * Sets whether this game nodes content should be preserved even in the case
     * of compacting the game tree.
     */
    public void setPreserve(boolean preserve);

    /**
     * Returns whether this node is a terminal node or not.
     */
    public boolean isTerminal();
    
}
