package org.eclipse.palamedes.gdl.core.model;

import java.util.List;

/**
 * IGameState provides an interface for general interaction with a single game 
 * state. 
 * 
 * Usually it is a Wrapper around a state object which was given by the games 
 * reasoner. AbstractState provides some basic functionality for such a wrapper.
 * 
 * @see org.eclipse.palamedes.gdl.core.model.utils.AbstractState  
 */
public interface IGameState {

    /**
     * Returns the combined legal moves.
     */
    public List<IMove[]> getCombinedLegalMoves();

    /**
     * Returns an array of the legal moves of each player. If not already resolved the method returns null.
     * 
     */
    public IMove[][] getLegalMoves();

    /**
     * Returns the goal value for the player specified by roleIndex of the state
     * if the state is a goal state; otherwise -1.
     */
    public int getGoalValue(int roleIndex);

    /**
     * Returns the goal values in the same order as IGame.getRoleNames()
     * @see org.eclipse.palamedes.gdl.core.model.IGame#getRoleNames()
     */
    public int[] getGoalValues();

    /**
     * Returns true if the state is a terminal state; otherwise false.
     */
    public boolean isTerminal();

    /**
     * Sets the goal value of the game state. If the value is unknown it should
     * get a -1.
     */
    public void setGoalValues(int[] goalValues);

    /**
     * Sets the terminal flag of the state.
     */
    public void setTerminal();

    /**
     * Sets the legal moves for this game state.
     */
    public void setLegalMoves(IMove[][] moves);

    /**
     * Returns the fluents with the given name.
     */
    public List<String> getFluentNames();

    /**
     * Returns the fluents with the given name.
     */
    public List<IFluent> getFluentByName(String name);

    /**
     * Returns a list containing all fluents of the game state.
     */
    public List<IFluent> getFluents();

    /**
     * Returns the state object that was provided by the reasoner.
     */
    public Object getNativeState();
}
