/*
    Copyright (C) 2008 Stephan Schiffel <stephan.schiffel@gmx.de>

    This file is part of Palamedes-IDE.

    Palamedes-IDE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Palamedes-IDE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Palamedes-IDE.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.eclipse.palamedes.gdl.core.resolver.jocular;

import org.eclipse.palamedes.gdl.core.model.utils.TermWrapper;
import org.eclipse.palamedes.kif.core.ast.AST;
import org.eclipse.palamedes.kif.core.ast.KIFSeq;
import org.eclipse.palamedes.kif.core.ast.KIFTerm;

import stanfordlogic.gdl.SymbolTable;
import stanfordlogic.prover.TermFunction;
import stanfordlogic.prover.TermObject;
import stanfordlogic.prover.TermVariable;
import stanfordlogic.prover.Term;

public class TermAdapter extends TermWrapper<Term> {
	private SymbolTable symbolTable;
	
	public TermAdapter(SymbolTable symbolTable, Term expr){
		super(expr);
		this.symbolTable=symbolTable;
	}

	@Override
	public KIFTerm getKIFTerm() {
		return getKIFTerm(symbolTable, nativeTerm);
	}
	
	public static final KIFTerm getKIFTerm(SymbolTable symbolTable, Term t){
		if(t instanceof TermVariable){
			return new AST().newIndVar(symbolTable.get(((TermVariable)t).getName()));
		}else if(t instanceof TermObject){
			return new AST().newObjConst(symbolTable.get(((TermObject)t).getToken()));
		}else if(t instanceof TermFunction){
			TermFunction f=(TermFunction)t;
			KIFSeq<KIFTerm> args=new KIFSeq<KIFTerm>();
			for(int i=0;i<f.getArity();i++){
				args.add(getKIFTerm(symbolTable, f.getTerm(i)));
			}
			return new AST().newFunTerm(symbolTable.get(f.getName()), args);
		}
		return null;
	}
}
