package org.eclipse.palamedes.gdl.core.simulation.strategies;

import java.util.BitSet;
import java.util.List;
import java.util.Random;

import org.eclipse.palamedes.gdl.core.model.IGameNode;
import org.eclipse.palamedes.gdl.core.model.IMove;
import org.eclipse.palamedes.gdl.core.model.utils.Game;


public class FringeFrame {

    /** A random number generator. */
    protected static final Random random = new Random();

    final IGameNode node;
    final BitSet    movesForExpansion; // true means already visited

    /** 
     * Stores the combined move list. 
     * The list has to be stored here since some reasoner regenerate the moves 
     * in indeterministic order.
     */
    final List<IMove[]> combinedMoves;

    public FringeFrame (IGameNode gameNode) {
        combinedMoves = gameNode.getState().getCombinedLegalMoves();
        if ( combinedMoves.isEmpty() ) {
            System.err.println(Game.showGameNodePath(gameNode));
            throw new IllegalArgumentException("Error: No moves available!");
        }
        node              = gameNode;
        movesForExpansion = new BitSet(combinedMoves.size());
        movesForExpansion.set(0, combinedMoves.size(), true);
    }

    /**
     * Returns the index of an unexpanded move and flips the corresponding
     * marker bit.If no more moves are available the method returns -1.
     *
     * Beware: Each index is only returned once!
     */
    private int getRandomUnexpandedMoveIndex() {

        // find the index for the selected move
        int moveNumber = random.nextInt(movesForExpansion.cardinality());
        int index      = movesForExpansion.nextSetBit(0);
        for (int i = 0; i < moveNumber; i++ )
            index = movesForExpansion.nextSetBit(index + 1);
        
        // mark move as chosen
        movesForExpansion.set(index, false);

        // return its index
        return index;
    }
    
    public IMove[] getRandomUnexpandedMove() {        
        return !movesForExpansion.isEmpty() ?
                              combinedMoves.get(getRandomUnexpandedMoveIndex()):
                              null;        
    }
    
    public boolean hasUnexpandedMove() {
        return !movesForExpansion.isEmpty();
    }

}
